package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.DeleteBasicExamStudentStatusDto;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.params.BasicExamStudentParam;
import com.qmth.distributed.print.business.bean.query.BasicExamStudentPageQuery;
import com.qmth.distributed.print.business.bean.query.BasicExamStudentQuery;
import com.qmth.distributed.print.business.bean.result.BasicExamStudentResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.teachcloud.common.bean.sync.TSyncDataStudent;
import com.qmth.teachcloud.common.enums.ExamNumberStyleEnum;
import com.qmth.distributed.print.business.enums.RequiredFieldsEnum;
import com.qmth.distributed.print.business.mapper.BasicExamStudentMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncBasicExamStudentExportService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableDisabledValue;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.dto.mark.AbInfoVo;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.mark.service.MarkTaskService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 考生字典表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
@Service
public class BasicExamStudentServiceImpl extends ServiceImpl<BasicExamStudentMapper, BasicExamStudent> implements BasicExamStudentService {

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private BasicExamRuleService basicExamRuleService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private TeachCourseService teachCourseService;
    @Resource
    private CommonCacheService commonCacheService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private BasicExamService basicExamService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    private ExamStudentService examStudentService;
    @Resource
    private ExamCardModelFourService examCardModelFourService;
    @Resource
    private TBTaskService tbTaskService;
    @Resource
    private AsyncBasicExamStudentExportService asyncBasicExamStudentExportService;

    @Override
    public IPage<BasicExamStudent> queryPage(BasicExamStudentPageQuery query) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(requestUser.getSchoolId(), requestUser.getId(),
                ServletUtil.getRequest().getServletPath());
        query.setSchoolId(requestUser.getSchoolId());
        return this.baseMapper.findBasicExamStudentPage(new Page<>(query.getPageNumber(), query.getPageSize()), query, dpr);
    }

    @Override
    public List<BasicExamStudent> exportList(BasicExamStudentPageQuery query, SysUser sysUser, Long privilegeId) {
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(privilegeId, sysUser);
        return this.baseMapper.findBasicExamStudentPage(query, dpr);
    }

    @Transactional
    @Override
    public Long save(BasicExamStudentParam basicExamStudentParam, SysUser requestUser) {
        BasicExam basicExam = basicExamService.getById(basicExamStudentParam.getExamId());
        BasicExamStudent basicExamStudent = this.editEntityHelp(basicExamStudentParam, basicExam, requestUser);
        this.saveOrUpdate(basicExamStudent);
        // 更新扫描阅卷数据
        this.updateMarkData(basicExam.getId(), Arrays.asList(basicExamStudent), requestUser.getId());
        // 自动创建课程管理数据
        teachCourseService.batchCreateTeachCourse(basicExamStudent.getSchoolId(), basicExamStudent.getExamId(), Arrays.asList(basicExamStudent));
        return basicExamStudent.getId();
    }

    @Override
    public void exportLogic(BasicExamStudentPageQuery query) {
        Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.BASIC_EXAM_STUDENT_EXPORT, query.getSemesterId(), query.getExamId(), (SysUser) ServletUtil.getRequestUser());
        map.put("query", query);
        map.put("privilegeId", ServletUtil.getCurrentPrivilegeId());
        asyncBasicExamStudentExportService.exportTask(map);
    }

    @Transactional
    @Override
    public void deleteBatch(List<Long> idList) {
        if (CollectionUtils.isNotEmpty(idList)) {
            List<BasicExamStudent> basicExamStudentList = this.listByIds(idList);
            List<Long> basicExamStudentIds = basicExamStudentList.stream().map(BasicExamStudent::getId).collect(Collectors.toList());
            List<MarkStudent> markStudentList = markStudentService.listByBasicStudentIds(basicExamStudentIds);
            if (CollectionUtils.isNotEmpty(markStudentList)) {
                for (MarkStudent markStudent : markStudentList) {
                    if (markTaskService.countByStudentId(markStudent.getId()) > 0) {
                        BasicCourse basicCourse = basicCourseService.getById(markStudent.getCourseId());
                        throw ExceptionResultEnum.ERROR.exception("课程[" + basicCourse.getName() + "(" + basicCourse.getCode() + ")]，学号[" + markStudent.getStudentCode() + "]考生已有评卷任务，不允许删除");
                    }
                }
            }

            if (CollectionUtils.isNotEmpty(basicExamStudentIds)) {
                examStudentService.deleteByBasicStudentId(basicExamStudentIds);
                markStudentService.deleteByBasicStudentId(basicExamStudentIds);
            }

            Set<String> paperNumberSet = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).map(BasicExamStudent::getPaperNumber).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(paperNumberSet)) {
                Long examId = basicExamStudentList.get(0).getExamId();
                for (String paperNumber : paperNumberSet) {
                    MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
                    if (markPaper == null) {
                        continue;
                    }
                    // 更新扫描数据
                    long count = markStudentService.countByExamIdAndPaperNumber(examId, paperNumber);
                    if (count == 0) {
                        markPaperService.deleteByExamIdAndPaperNumber(examId, paperNumber);
                    } else {
                        markPaperService.updateStudentCountByExamIdAndPaperNumber(examId, paperNumber);
                        markPaperService.updateUploadCount(examId, paperNumber, markStudentService.countUploadedByExamIdAndPaperNumber(examId, paperNumber));
                        markPaperService.updateAbsentCount(examId, paperNumber, markStudentService.countAbsentByExamIdAndPaperNumber(examId, paperNumber));
                    }

                    // 考务数据全删除时，将卷型更新为未曝光
                    if (examStudentService.countByExamIdAndPaperNumber(examId, paperNumber) == 0) {
                        examTaskDetailService.updateUnexposePaperType(examId, paperNumber, markPaper.getSerialNumber());
                    }
                }
            }
            this.removeByIds(idList);
        }
    }

    /**
     * 构造考生新增/编辑对象
     *
     * @param basicExamStudentParam 考生参数
     * @param requestUser           请求人
     * @return 考生对象
     */
    @Override
    public BasicExamStudent editEntityHelp(BasicExamStudentParam basicExamStudentParam, BasicExam basicExam, SysUser requestUser) {
        List<String> errorMsgList = new ArrayList<>();
        Long schoolId = requestUser.getSchoolId();
        Long requestUserId = requestUser.getId();

        String studentCode = basicExamStudentParam.getStudentCode();
        String paperNumber = basicExamStudentParam.getPaperNumber();
        String teacherName = basicExamStudentParam.getTeacherName();
        String teacherCode = basicExamStudentParam.getTeacherCode();
        String paperType = basicExamStudentParam.getPaperType();

        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(schoolId);
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
        }

        if (StringUtils.isNotBlank(studentCode) && !studentCode.matches(SystemConstant.REGULAR_EXPRESSION_OF_STUDENT_CODE)) {
            throw ExceptionResultEnum.ERROR.exception("学号只能由大写字母、数字组成");
        }

        Long teacherId = null;
        if (SystemConstant.strNotNull(teacherCode) && !SystemConstant.strNotNull(teacherName)) {
            errorMsgList.add("任课老师必填");
        }
        if (SystemConstant.strNotNull(teacherName) && !SystemConstant.strNotNull(teacherCode)) {
            errorMsgList.add("任课老师工号必填");
        }
        if (StringUtils.isNotBlank(teacherCode) && StringUtils.isNotBlank(teacherName)) {
            SysUser sysUser = sysUserService.getByLoginName(schoolId, teacherCode);
            if (sysUser == null) {
                errorMsgList.add("任课老师工号[" + teacherCode + "]用户不存在");
            } else {
                if (!sysUser.getEnable()) {
                    errorMsgList.add("任课老师工号[" + teacherCode + "]用户已禁用");
                } else if (!sysUser.getRealName().equals(teacherName)) {
                    errorMsgList.add("任课老师[" + teacherName + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                } else {
                    teacherId = sysUser.getId();
                    // 校验同课程、同教学班下任课老师必须相同
                    List<BasicExamStudent> basicExamStudentList = this.listByExamIdAndCourseIdAndTeachClassName(basicExamStudentParam.getExamId(), basicExamStudentParam.getCourseId(), basicExamStudentParam.getTeachClassName());
                    if (CollectionUtils.isNotEmpty(basicExamStudentList)) {
                        long count = basicExamStudentList.stream().filter(m -> m.getTeacherId() != null && !sysUser.getId().equals(m.getTeacherId())).count();
                        if (count > 0) {
                            throw ExceptionResultEnum.ERROR.exception("教学班[" + basicExamStudentParam.getTeachClassName() + "]存在其它任课老师，教学班的任课老师必须相同");
                        }
                    }
                }
            }
        }

        // 校验试卷编号存在时，任课教师，考试时间，考点考场均必填；
        BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
        boolean needPaperNumber = (basicSchool.getHasPaperNumber() != null && basicSchool.getHasPaperNumber()) || ExamModelEnum.MODEL4.equals(basicExam.getExamModel());
        if ((needPaperNumber && StringUtils.isBlank(paperNumber)) || (StringUtils.isBlank(paperNumber) && StringUtils.isNotBlank(paperType))) {
            errorMsgList.add("试卷编号必填");
        }

        if (CollectionUtils.isNotEmpty(errorMsgList)) {
            throw ExceptionResultEnum.ERROR.exception(String.join(";", errorMsgList));
        }

        // 校验试卷编号和卷型
        if (!ExamModelEnum.MODEL4.equals(basicExam.getExamModel()) && StringUtils.isNoneBlank(basicExamStudentParam.getPaperNumber(), basicExamStudentParam.getPaperType())) {
            // 默认取备用卷1的卷型（所有备用卷的卷型一样，A或者AB）
            ExamTask examTask = examTaskService.getByExamIdAndPaperNumber(basicExamStudentParam.getExamId(), basicExamStudentParam.getPaperNumber());
            if (examTask == null) {
                throw ExceptionResultEnum.ERROR.exception("试卷编号[" + basicExamStudentParam.getPaperNumber() + "]命题任务不存在");
            }
            ExamTaskDetail examTaskDetail;
            if (examTask.getOpenAb()) {
                examTaskDetail = examTaskDetailService.getByExamIdAndPaperNumberAndSerialNumber(basicExamStudentParam.getExamId(), basicExamStudentParam.getPaperNumber(), 1);
                if (examTaskDetail != null) {
                    if (!examTaskDetail.getPaperType().contains(basicExamStudentParam.getPaperType())) {
                        throw ExceptionResultEnum.ERROR.exception("试卷编号[" + basicExamStudentParam.getPaperNumber() + "]的命题任务卷型为[" + examTaskDetail.getPaperType() + "]");
                    }
                }
            } else {
                List<ExamTaskDetail> examTaskDetails = examTaskDetailService.listByExamTaskId(examTask.getId());
                examTaskDetail = examTaskDetails.stream().filter(m -> m.getPaperType().equals(basicExamStudentParam.getPaperType())).findFirst().orElse(null);
                if (examTaskDetail == null) {
                    throw ExceptionResultEnum.ERROR.exception("试卷编号[" + basicExamStudentParam.getPaperNumber() + "]的命题任务不存在卷型[" + basicExamStudentParam.getPaperType() + "]");

                }
            }
        }

        BasicExamStudent basicExamStudent = new BasicExamStudent();
        BeanUtils.copyProperties(basicExamStudentParam, basicExamStudent);
        basicExamStudent.setSchoolId(schoolId);
        basicExamStudent.setTeacherId(teacherId);

        if (ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
            if (basicExamStudent.getPaperType() == null) {
                basicExamStudent.setPaperType(SystemConstant.DEFAULT_PAPER_TYPE_A);
            } else if (!SystemConstant.DEFAULT_PAPER_TYPE_A.equals(basicExamStudent.getPaperType())) {
                throw ExceptionResultEnum.ERROR.exception("模式4的考生卷型只能为A卷");
            }
        }

        if (SystemConstant.longNotNull(basicExamStudent.getId())) {
            // 编辑 （学号不可更改）
            BasicExamStudent dbBasicExamStudent = this.getById(basicExamStudent.getId());
            if (Objects.isNull(dbBasicExamStudent)) {
                throw ExceptionResultEnum.ERROR.exception("未查询到此考生");
            }
            if (!basicExamStudent.getStudentCode().equals(dbBasicExamStudent.getStudentCode())) {
                throw ExceptionResultEnum.ERROR.exception("不允许修改学号");
            }
            if (StringUtils.isNotBlank(dbBasicExamStudent.getPaperNumber()) && !dbBasicExamStudent.getPaperNumber().equals(paperNumber)) {
                throw ExceptionResultEnum.ERROR.exception("不允许修改试卷编号");
            }
            if (StringUtils.isNotBlank(dbBasicExamStudent.getPaperType()) && !dbBasicExamStudent.getPaperType().equals(paperType)) {
                throw ExceptionResultEnum.ERROR.exception("不允许修改卷型");
            }

            basicExamStudent.updateInfo(requestUserId);
        } else {
            // 新增 （学号在考试课程下唯一）
            if (this.count(new QueryWrapper<BasicExamStudent>().lambda()
                    .eq(BasicExamStudent::getExamId, basicExamStudent.getExamId()).eq(BasicExamStudent::getCourseId, basicExamStudent.getCourseId())
                    .eq(BasicExamStudent::getStudentCode, basicExamStudent.getStudentCode())) > 0) {
                throw ExceptionResultEnum.ERROR.exception(String.format("学号[%s]考生已存在", basicExamStudent.getStudentCode()));
            }
            basicExamStudent.insertInfo(requestUserId);
        }

        // 模式4必填字段校验
        if (ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
            if (StringUtils.isBlank(basicExamStudent.getPaperNumber())) {
                throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.PAPER_NUMBER.getName() + "必填");
            }
            if (StringUtils.isBlank(basicExamStudent.getStudentCode())) {
                throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.STUDENT_CODE.getName() + "必填");
            }
            if (StringUtils.isBlank(basicExamStudent.getStudentName())) {
                throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.STUDENT_NAME.getName() + "必填");
            }
            if (basicExamStudent.getCourseId() == null) {
                throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.COURSE_CODE.getName() + "必填");
            }
            if (StringUtils.isBlank(basicExamStudent.getTeachClassName())) {
                throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.TEACH_CLASS_NAME.getName() + "必填");
            }
        }

        // 试卷编号绑定其它课程
        if (StringUtils.isNotBlank(basicExamStudent.getPaperNumber())) {
//            BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(schoolId);
//            Optional<EnumResult> optional = basicExamRule.getRequiredFieldList().stream().filter(m -> RequiredFieldsEnum.EXAM_PLACE.getCode().equals(m.getCode()) && m.getEnable()).findFirst();
//            // 试卷编号不为空时，校区（考点）必填
//            if (optional.isPresent() && optional.get().getEnable() && StringUtils.isBlank(basicExamStudent.getExamPlace())) {
//                throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.EXAM_PLACE.getName() + "必填");
//            }
            if (basicExamStudent.getExamStartTime() != null && basicExamStudent.getExamEndTime() != null) {
                QueryWrapper<BasicExamStudent> queryWrapper = new QueryWrapper<>();
                LambdaQueryWrapper<BasicExamStudent> lambda = queryWrapper.lambda();
                lambda.eq(BasicExamStudent::getExamId, basicExamStudent.getExamId()).eq(BasicExamStudent::getPaperNumber, basicExamStudent.getPaperNumber())
                        .and(m -> m.ne(BasicExamStudent::getExamStartTime, basicExamStudent.getExamStartTime()).or().ne(BasicExamStudent::getExamEndTime, basicExamStudent.getExamEndTime()));
                if (basicExamStudent.getId() != null) {
                    lambda.ne(BasicExamStudent::getId, basicExamStudent.getId());
                }
                if (this.count(queryWrapper) > 0) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]存在不同考试时间", basicExamStudent.getPaperNumber()));
                }
            }

            if (this.count(new QueryWrapper<BasicExamStudent>().lambda()
                    .eq(BasicExamStudent::getExamId, basicExamStudent.getExamId()).eq(BasicExamStudent::getPaperNumber, basicExamStudent.getPaperNumber())
                    .ne(BasicExamStudent::getCourseId, basicExamStudent.getCourseId())) > 0) {
                throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]已绑定其它课程", basicExamStudent.getPaperNumber()));
            }
        }

        if (basicExamStudent.getExamStartTime() != null && basicExamStudent.getExamEndTime() != null) {
            Map<String, String> dateMap = ConvertUtil.analyzeDateAndTime(basicExamStudent.getExamStartTime(), basicExamStudent.getExamEndTime());
            String date = dateMap.get("date");
            String time = dateMap.get("time");
            basicExamStudent.setExamDate(date);
            basicExamStudent.setExamTime(time);
        }

        parserRequiredField(basicExamStudent, basicExamRule);
        return basicExamStudent;
    }

    private void parserRequiredField(BasicExamStudent basicExamStudent, BasicExamRule basicExamRule) {
        // 组装requiredFields。extendFields前端传
        Map<String, EnumResult> enumResultMap = basicExamRule.getRequiredFieldList().stream().filter(m -> m.getEnable()).collect(Collectors.toMap(EnumResult::getCode, m -> m));
        List<CodeNameEnableDisabledValue> requiredFields = new ArrayList<>();
        Field[] declaredFields = basicExamStudent.getClass().getDeclaredFields();
        try {
            boolean examDateHasValue = false;
            for (Field declaredField : declaredFields) {
                declaredField.setAccessible(true);
                String fieldName = declaredField.getName();
                if ("courseId".equals(fieldName)) {
                    BasicCourse basicCourse = basicCourseService.getById(basicExamStudent.getCourseId());
                    if (basicCourse == null) {
                        throw ExceptionResultEnum.ERROR.exception("课程不存在");
                    }
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.COURSE_CODE.getCode(), RequiredFieldsEnum.COURSE_CODE.getName(), RequiredFieldsEnum.COURSE_CODE.getEnable(), RequiredFieldsEnum.COURSE_CODE.getDisabled(), basicCourse.getCode()));
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.COURSE_NAME.getCode(), RequiredFieldsEnum.COURSE_NAME.getName(), RequiredFieldsEnum.COURSE_NAME.getEnable(), RequiredFieldsEnum.COURSE_NAME.getDisabled(), basicCourse.getName()));
                    SysOrg sysOrg = sysOrgService.getById(basicCourse.getTeachingRoomId());
                    if (sysOrg == null) {
                        throw ExceptionResultEnum.ERROR.exception("课程开课学院不存在");
                    }
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.TEACHING_ROOM_NAME.getCode(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getName(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getEnable(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getDisabled(), sysOrg.getName()));
                } else if ("teacherId".equals(fieldName)) {
                    String loginName = null;
                    String realName = null;
                    if (basicExamStudent.getTeacherId() != null) {
                        SysUser sysUser = sysUserService.getById(basicExamStudent.getTeacherId());
                        if (sysUser == null) {
                            throw ExceptionResultEnum.ERROR.exception("任课老师不存在");
                        }
                        loginName = sysUser.getLoginName();
                        realName = sysUser.getRealName();
                    }
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.TEACHER_CODE.getCode(), RequiredFieldsEnum.TEACHER_CODE.getName(), RequiredFieldsEnum.TEACHER_CODE.getEnable(), RequiredFieldsEnum.TEACHER_CODE.getDisabled(), loginName));
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.TEACHER_NAME.getCode(), RequiredFieldsEnum.TEACHER_NAME.getName(), RequiredFieldsEnum.TEACHER_NAME.getEnable(), RequiredFieldsEnum.TEACHER_NAME.getDisabled(), realName));

                } else if ("examStartTime".equals(fieldName) || "examEndTime".equals(fieldName)) {
                    Long examStartTime = basicExamStudent.getExamStartTime();
                    Long examEndTime = basicExamStudent.getExamEndTime();
                    if (SystemConstant.longNotNull(examStartTime) && SystemConstant.longNotNull(examEndTime) && !examDateHasValue) {
                        Map<String, String> dateMap = ConvertUtil.analyzeDateAndTime(examStartTime, examEndTime);
                        String date = dateMap.get("date");
                        String time = dateMap.get("time");
                        requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.EXAM_DATE.getCode(), RequiredFieldsEnum.EXAM_DATE.getName(), RequiredFieldsEnum.EXAM_DATE.getEnable(), RequiredFieldsEnum.EXAM_DATE.getDisabled(), date));
                        requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.EXAM_TIME.getCode(), RequiredFieldsEnum.EXAM_TIME.getName(), RequiredFieldsEnum.EXAM_TIME.getEnable(), RequiredFieldsEnum.EXAM_TIME.getDisabled(), time));
                        examDateHasValue = true;
                    }
                } else {
                    if (enumResultMap.containsKey(fieldName)) {
                        requiredFields.add(new CodeNameEnableDisabledValue(fieldName, enumResultMap.get(fieldName).getName(), enumResultMap.get(fieldName).getEnable(), enumResultMap.get(fieldName).getDisabled(), declaredField.get(basicExamStudent) != null ? declaredField.get(basicExamStudent).toString() : ""));
                    }
                }
            }
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
        basicExamStudent.setRequiredFields(JSON.toJSONString(requiredFields));
    }

    @Override
    public List<BasicExamStudent> listByExamIdAndCourseIdAndPaperNumber(Long examId, Long courseId, String paperNumber) {
        QueryWrapper<BasicExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicExamStudent::getExamId, examId)
                .eq(BasicExamStudent::getCourseId, courseId)
                .eq(BasicExamStudent::getPaperNumber, paperNumber)
                .orderByAsc(BasicExamStudent::getStudentCode);
        return this.list(queryWrapper);
    }

    @Override
    public List<BasicExamStudentResult> listByParams(BasicExamStudentQuery basicExamStudentQuery) {
        return this.baseMapper.listByParams(basicExamStudentQuery);
    }

    @Override
    public boolean updateStatus(Long id, BasicExamStudentStatusEnum status) {
        return this.update(new UpdateWrapper<BasicExamStudent>().lambda().set(BasicExamStudent::getStatus, status).eq(BasicExamStudent::getId, id));
    }

    @Transactional
    @Override
    public void updateTeacherIdById(List<BasicExamStudent> basicExamStudentList) {
        for (BasicExamStudent basicExamStudent : basicExamStudentList) {
            this.update(new UpdateWrapper<BasicExamStudent>().lambda()
                    .set(BasicExamStudent::getTeacherId, basicExamStudent.getTeacherId())
                    .set(BasicExamStudent::getRequiredFields, basicExamStudent.getRequiredFields())
                    .eq(BasicExamStudent::getId, basicExamStudent.getId()));
        }
    }

    @Override
    public void updatePaperNumberAndPaperTypeById(List<ExamStudent> examStudentList, BasicCardRule basicCardRule, Boolean openAb) {
        for (ExamStudent examStudent : examStudentList) {
            BasicExamStudent basicExamStudent = this.getById(examStudent.getBasicStudentId());
            if (StringUtils.isNoneBlank(basicExamStudent.getPaperNumber(), basicExamStudent.getPaperType()) && !examStudent.getPaperNumber().equals(basicExamStudent.getPaperNumber())) {
                throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]，学号[%s]考生在试卷编号[%s]生成了印刷任务", examStudent.getPaperNumber(), basicExamStudent.getStudentCode(), basicExamStudent.getPaperNumber()));
            }
            List<CodeNameEnableValue> requiredFieldList = basicExamStudent.getRequiredFieldList();
            for (CodeNameEnableValue codeNameEnableValue : requiredFieldList) {
                if (RequiredFieldsEnum.PAPER_NUMBER.getCode().equals(codeNameEnableValue.getCode())) {
                    codeNameEnableValue.setValue(examStudent.getPaperNumber());
                }
            }
            //
            this.update(new UpdateWrapper<BasicExamStudent>().lambda()
                    .set(BasicExamStudent::getPaperNumber, examStudent.getPaperNumber())
                    .set(ExamNumberStyleEnum.PRINT.equals(basicCardRule.getExamNumberStyle()) || (!openAb && ExamNumberStyleEnum.FILL.equals(basicCardRule.getExamNumberStyle())), BasicExamStudent::getPaperType, examStudent.getPaperType())
                    .set(BasicExamStudent::getRequiredFields, JSON.toJSONString(requiredFieldList))
                    .eq(BasicExamStudent::getId, examStudent.getBasicStudentId()));
        }
    }

    @Override
    public void updateMarkData(Long examId, List<BasicExamStudent> basicExamStudentList, Long createId) {
        BasicExam basicExam = basicExamService.getById(examId);
        Map<String, List<BasicExamStudent>> stringMap = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).collect(Collectors.groupingBy(BasicExamStudent::getPaperNumber));
        for (Map.Entry<String, List<BasicExamStudent>> entry : stringMap.entrySet()) {
            BasicExamStudent basicExamStudent = entry.getValue().get(0);
            AbInfoVo abInfoVo = markStudentService.findExamTaskPaperTypeOpenStatus(examId, entry.getKey());
            boolean containPaperType = abInfoVo != null && (ExamNumberStyleEnum.PRINT.equals(abInfoVo.getExamNumberStyle()) || (ExamNumberStyleEnum.FILL.equals(abInfoVo.getExamNumberStyle()) && !abInfoVo.getOpenAb()));
            MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, entry.getKey());
            if (markPaper == null && ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
                String paperType = SystemConstant.DEFAULT_PAPER_TYPE_A;
                markPaper = new MarkPaper(examId, basicExamStudent.getCourseId(), entry.getKey(), String.valueOf(SystemConstant.getDbUuid()), createId, 1, paperType);
                markPaperService.save(markPaper);

                // 保存模式4题卡
                examCardModelFourService.saveQuestionAndAnswerCard(null, markPaper, createId);
            }
            if (markPaper != null) {
                List<MarkStudent> markStudentList = new ArrayList<>();
                Set<String> secretNumberSet = new HashSet<>();
                MarkPaper finalMarkPaper = markPaper;
                entry.getValue().forEach(m -> {
                    MarkStudent markStudent = markStudentService.assembleByBasicExamStudent(m, finalMarkPaper, secretNumberSet, containPaperType);
                    if (markStudent != null) {
                        markStudentList.add(markStudent);
                    }
                });
                if (CollectionUtils.isNotEmpty(markStudentList)) {
                    markStudentService.saveOrUpdateBatch(markStudentList);
                    markPaperService.updateStudentCountByExamIdAndPaperNumber(examId, entry.getKey());
                    markPaperService.updateUploadCount(examId, entry.getKey(), markStudentService.countUploadedByExamIdAndPaperNumber(examId, entry.getKey()));
                }
            }
        }
    }

    @Override
    public List<DeleteBasicExamStudentStatusDto> getStudentMarkStatus(List<Long> idList) {
        List<BasicExamStudent> basicExamStudentList = this.listByIds(idList);
        List<DeleteBasicExamStudentStatusDto> list = new ArrayList<>();
        for (BasicExamStudent basicExamStudent : basicExamStudentList) {
            List<MarkStudent> markStudentList = markStudentService.listByStudentId(basicExamStudent.getId());
            Map<Long, List<MarkStudent>> map = markStudentList.stream().collect(Collectors.groupingBy(MarkStudent::getBasicStudentId));
            for (Map.Entry<Long, List<MarkStudent>> entry : map.entrySet()) {
                DeleteBasicExamStudentStatusDto deleteBasicExamStudentStatusDto = new DeleteBasicExamStudentStatusDto(basicExamStudent.getStudentName(), basicExamStudent.getStudentCode());
                deleteBasicExamStudentStatusDto.setScanStatus(entry.getValue().stream().filter(m -> m.getUpload() != null && m.getUpload()).count() > 0);
                int taskCount = 0;
                for (MarkStudent markStudent : entry.getValue()) {
                    if (taskCount > 0) {
                        break;
                    }
                    taskCount += markTaskService.countByStudentId(markStudent.getId());
                }
                deleteBasicExamStudentStatusDto.setMarkTaskStatus(taskCount > 0);
                list.add(deleteBasicExamStudentStatusDto);
            }
        }
        return list;
    }

    /**
     * 查询班级和老师信息
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @return
     */
    @Override
    public ExamStudentDto queryExamStudent(Long examId, Long courseId, String paperNumber) {
        return this.baseMapper.queryExamStudent(examId, courseId, paperNumber);
    }

    @Override
    public List<BasicExamStudent> listByExamIdAndCourseIdAndTeachClassName(Long examId, Long courseId, String teachClassName) {
        QueryWrapper<BasicExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicExamStudent::getExamId, examId)
                .eq(BasicExamStudent::getCourseId, courseId)
                .eq(BasicExamStudent::getTeachClassName, teachClassName);
        return this.list(queryWrapper);
    }

    @Override
    public void clearPaperNumberAndPaperTypeById(List<Long> ids) {
        UpdateWrapper<BasicExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicExamStudent::getPaperType, null)
                .in(BasicExamStudent::getId, ids);
        this.update(updateWrapper);
    }

    @Override
    public void clearPaperTypeByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<BasicExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicExamStudent::getPaperType, null)
                .eq(BasicExamStudent::getExamId, examId)
                .eq(BasicExamStudent::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public void clearPaperNumberAndPaperTypeByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<BasicExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicExamStudent::getPaperNumber, null)
                .set(BasicExamStudent::getPaperType, null)
                .eq(BasicExamStudent::getExamId, examId)
                .eq(BasicExamStudent::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public Map<String, Object> saveBasicExamStudentFormSync(Long schoolId, Long examId, List<TSyncDataStudent> examStudentDataVoList) {
        BasicExam basicExam = basicExamService.getById(examId);

        List<BasicExamStudent> basicExamStudentList = this.list(new QueryWrapper<BasicExamStudent>().lambda()
                .select(BasicExamStudent::getId, BasicExamStudent::getCourseId, BasicExamStudent::getStudentCode, BasicExamStudent::getPaperNumber, BasicExamStudent::getPaperType, BasicExamStudent::getExamStartTime, BasicExamStudent::getExamEndTime)
                .eq(BasicExamStudent::getSchoolId, schoolId).eq(BasicExamStudent::getExamId, examId));
        Map<String, BasicExamStudent> courseIdStudentCodeMap = basicExamStudentList.stream().collect(Collectors.toMap(k -> k.getCourseId() + SystemConstant.HYPHEN + k.getStudentCode(), Function.identity(), (v1, v2) -> v1));

        BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(schoolId);
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
        }
        // 学校设置强制校验卷型或者模式4
        boolean needPaperNumber = (basicSchool.getHasPaperNumber() != null && basicSchool.getHasPaperNumber());
        // 试卷编号->课程
        Map<String, Long> paperNumberCourseIdMap = new HashMap<>();
        // 试卷编号->考试时间
        Map<String, String> paperNumberExamTimeMap = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).collect(Collectors.toMap(BasicExamStudent::getPaperNumber, m -> m.getExamStartTime() + "-" + m.getExamEndTime(), (v1, v2) -> v1));
        Map<String, Long> basicCourseIdMap = new HashMap<>();
        // 课程id+教学班名称->任课老师Id
        Map<String, Long> courseTeachClassTeacherMap = new HashMap<>();
        // 课程学号唯一
        List<String> courseCodeStudentCodeList = new ArrayList<>();

        List<BasicExamStudent> basicExamStudents = new ArrayList<>();

        StringJoiner errorMsg = new StringJoiner("<br>");
        AtomicInteger i = new AtomicInteger(0);
        for (TSyncDataStudent tSyncDataStudent : examStudentDataVoList) {
            StringJoiner stringJoiner = new StringJoiner("；<br>");
            try {
                if (StringUtils.isBlank(tSyncDataStudent.getCourseCode())) {
                    throw ExceptionResultEnum.ERROR.exception("课程代码必填");
                }
                if (StringUtils.isBlank(tSyncDataStudent.getCourseCollegeName())) {
                    throw ExceptionResultEnum.ERROR.exception("课程开课学院必填");
                }
                if (needPaperNumber && StringUtils.isBlank(tSyncDataStudent.getPaperNumber())) {
                    throw ExceptionResultEnum.ERROR.exception("试卷编号必填");
                }
                if (StringUtils.isBlank(tSyncDataStudent.getStudentName())) {
                    throw ExceptionResultEnum.ERROR.exception("姓名必填");
                } else if (StringUtils.isNotBlank(tSyncDataStudent.getStudentCode()) && !tSyncDataStudent.getStudentCode().matches(SystemConstant.REGULAR_EXPRESSION_OF_STUDENT_CODE)) {
                    throw ExceptionResultEnum.ERROR.exception("学号只能由大写字母、数字组成");
                }
                if (StringUtils.isBlank(tSyncDataStudent.getStudentCode())) {
                    throw ExceptionResultEnum.ERROR.exception("学号必填");
                }
                if (StringUtils.isBlank(tSyncDataStudent.getTeachClassName())) {
                    throw ExceptionResultEnum.ERROR.exception("教学班必填");
                }
                // 任课老师和任课老师工号都有值或都没值
                if (StringUtils.isNotBlank(tSyncDataStudent.getTeacherCode()) && StringUtils.isBlank(tSyncDataStudent.getTeacherName())) {
                    throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.TEACHER_NAME.getName() + "必填");
                }
                if (StringUtils.isNotBlank(tSyncDataStudent.getTeacherName()) && StringUtils.isBlank(tSyncDataStudent.getTeacherCode())) {
                    throw ExceptionResultEnum.ERROR.exception(RequiredFieldsEnum.TEACHER_CODE.getName() + "必填");
                }
                // 校验课程开课学院和课程代码
                Long courseId = null;
                String key = tSyncDataStudent.getCourseCode() + SystemConstant.HYPHEN + tSyncDataStudent.getCourseCollegeName();
                if (!basicCourseIdMap.containsKey(key)) {
                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, tSyncDataStudent.getCourseCollegeName());
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        throw ExceptionResultEnum.ERROR.exception("开课学院[" + tSyncDataStudent.getCourseCollegeName() + "]在学院层级不存在");
                    } else if (sysOrgList.size() == 1) {
                        SysOrg sysOrg = sysOrgList.get(0);
                        // 校验课程代码和开课学院
                        BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrg.getId(), tSyncDataStudent.getCourseCode());
                        if (Objects.isNull(basicCourse)) {
                            throw ExceptionResultEnum.ERROR.exception("课程代码[" + tSyncDataStudent.getCourseCode() + "]在开课学院[" + tSyncDataStudent.getCourseCollegeName() + "]不存在");
                        } else {
                            courseId = basicCourse.getId();
                            basicCourseIdMap.put(key, basicCourse.getId());
                        }
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("开课学院[" + tSyncDataStudent.getCourseCollegeName() + "]存在多个相同值");
                    }
                } else {
                    courseId = basicCourseIdMap.get(key);
                }

                if (courseId == null) {
                    throw ExceptionResultEnum.ERROR.exception("课程管理中未查到课程");
                }

                String courseStudentKey = courseId + SystemConstant.HYPHEN + tSyncDataStudent.getStudentCode();
                // 校验课程学号唯一
                if (courseCodeStudentCodeList.contains(courseStudentKey)) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("课程代码[%s]，开课学院[%s]，学号[%s]存在重复考生", tSyncDataStudent.getCourseCode(), tSyncDataStudent.getCourseCollegeName(), tSyncDataStudent.getStudentCode()));
                } else {
                    courseCodeStudentCodeList.add(courseStudentKey);
                }

                // 获取basicExamStudent对象
                BasicExamStudent basicExamStudent;
                if (courseIdStudentCodeMap.containsKey(courseStudentKey)) {
                    basicExamStudent = courseIdStudentCodeMap.get(courseStudentKey);
                } else {
                    basicExamStudent = new BasicExamStudent();
                    basicExamStudent.setId(SystemConstant.getDbUuid());
                    basicExamStudent.setSchoolId(schoolId);
                    basicExamStudent.setSemesterId(Long.valueOf(basicExam.getSemesterId()));
                    basicExamStudent.setExamId(examId);
                    basicExamStudent.setCourseId(courseId);
                    basicExamStudent.setStudentCode(tSyncDataStudent.getStudentCode());
                    basicExamStudent.setPaperNumber(tSyncDataStudent.getPaperNumber());
                }

                basicExamStudent.setStudentName(tSyncDataStudent.getStudentName());
                basicExamStudent.setSiteNumber(tSyncDataStudent.getSiteNumber());
                basicExamStudent.setCollegeName(tSyncDataStudent.getStudentCollegeName());
                basicExamStudent.setMajorName(tSyncDataStudent.getMajorName());
                basicExamStudent.setTeachClassName(tSyncDataStudent.getTeachClassName());
                basicExamStudent.setClassName(tSyncDataStudent.getClassName());
                basicExamStudent.setStatus(BasicExamStudentStatusEnum.N);
                basicExamStudent.setExamStartTime(tSyncDataStudent.getExamStartTime());
                basicExamStudent.setExamEndTime(tSyncDataStudent.getExamEndTime());
                basicExamStudent.setExamPlace(tSyncDataStudent.getExamPlace());
                basicExamStudent.setExamRoom(tSyncDataStudent.getExamRoom());

                if (StringUtils.isNotBlank(tSyncDataStudent.getPaperNumber())) {
                    if (paperNumberCourseIdMap.containsKey(tSyncDataStudent.getPaperNumber())) {
                        if (!paperNumberCourseIdMap.get(basicExamStudent.getPaperNumber()).equals(basicExamStudent.getCourseId())) {
                            throw ExceptionResultEnum.ERROR.exception("试卷编号[" + basicExamStudent.getPaperNumber() + "]不能绑定多个课程");
                        }
                    } else {
                        long count = basicExamStudentList.stream().filter(m -> m.getPaperNumber() != null && m.getPaperNumber().equals(tSyncDataStudent.getPaperNumber()) && !Objects.equals(m.getCourseId(), basicExamStudent.getCourseId())).count();
                        if (count > 0) {
                            throw ExceptionResultEnum.ERROR.exception("试卷编号[" + basicExamStudent.getPaperNumber() + "]已绑定其它课程");
                        } else {
                            paperNumberCourseIdMap.put(basicExamStudent.getPaperNumber(), basicExamStudent.getCourseId());
                        }
                    }

                    // 同一试卷编号只能对应同一考试时间
                    if (tSyncDataStudent.getExamStartTime() != null && tSyncDataStudent.getExamEndTime() != null) {
                        String examTimeValue = basicExamStudent.getExamStartTime() + "-" + basicExamStudent.getExamEndTime();
                        if (paperNumberExamTimeMap.containsKey(basicExamStudent.getPaperNumber())) {
                            if (!paperNumberExamTimeMap.get(basicExamStudent.getPaperNumber()).equals(examTimeValue)) {
                                throw ExceptionResultEnum.ERROR.exception("试卷编号[" + basicExamStudent.getPaperNumber() + "]只能有相同考试时间");
                            }
                        } else {
                            paperNumberExamTimeMap.put(basicExamStudent.getPaperNumber(), examTimeValue);
                        }
                    }

//                    basicExamStudent.setPaperNumber(tSyncDataStudent.getPaperNumber());
                }

//                if (ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
//                    if (basicExamStudent.getPaperType() == null) {
//                        basicExamStudent.setPaperType(SystemConstant.DEFAULT_PAPER_TYPE_A);
//                    }
//                }

                // 校验任课老师工号和姓名
                if (StringUtils.isNoneBlank(tSyncDataStudent.getTeacherCode(), tSyncDataStudent.getTeacherName())) {
                    SysUser sysUser = sysUserService.getByLoginName(schoolId, tSyncDataStudent.getTeacherCode());
                    if (sysUser == null) {
                        throw ExceptionResultEnum.ERROR.exception("任课老师工号[" + tSyncDataStudent.getTeacherCode() + "]用户不存在");
                    } else {
                        if (!sysUser.getEnable()) {
                            throw ExceptionResultEnum.ERROR.exception("任课老师工号[" + tSyncDataStudent.getTeacherCode() + "]用户已禁用");
                        } else if (!sysUser.getRealName().equals(tSyncDataStudent.getTeacherName())) {
                            throw ExceptionResultEnum.ERROR.exception("任课老师[" + tSyncDataStudent.getTeacherName() + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                        } else {
                            basicExamStudent.setTeacherId(sysUser.getId());
                        }
                    }

                    // 校验同课程下，同一教学班只能有一个任课老师
                    String courseTeachClassTeacherKey = basicExamStudent.getCourseId() + tSyncDataStudent.getTeachClassName();
                    if (courseTeachClassTeacherMap.containsKey(courseTeachClassTeacherKey)) {
                        Long teacherId = courseTeachClassTeacherMap.get(courseTeachClassTeacherKey);
                        if ((teacherId == null && basicExamStudent.getTeacherId() != null) || (teacherId != null && basicExamStudent.getTeacherId() == null) || (teacherId != null && !teacherId.equals(basicExamStudent.getTeacherId()))) {
                            throw ExceptionResultEnum.ERROR.exception("同课程同教学班，只允许设置一个任课老师");
                        }
                    } else {
                        courseTeachClassTeacherMap.put(courseTeachClassTeacherKey, basicExamStudent.getTeacherId());
                    }
                }
                parserRequiredField(basicExamStudent, basicExamRule);
                basicExamStudents.add(basicExamStudent);
            } catch (Exception e) {
                errorMsg.add(e.getMessage());
            }
        }

        this.saveOrUpdateBatch(basicExamStudents);
        // 更新删除考生启用状态为false
        this.baseMapper.updateEnableFalse(examId);
        this.updateMarkData(examId, basicExamStudentList, null);
        // 自动创建课程管理数据
        teachCourseService.batchCreateTeachCourse(schoolId, examId, basicExamStudentList);
        Map<String, Object> map = new HashMap<>();
        map.put("count", basicExamStudents.size());
        map.put("errMsg", errorMsg.toString());
        return map;
    }

    @Override
    public String listByTeacherByExamIdAndPaperNumber(Long examId, String paperNumber) {
        return this.baseMapper.listByTeacherByExamIdAndPaperNumber(examId, paperNumber);
    }

    @Override
    public BasicExamStudent findByExamIdAndCourseIdAndStudentCode(Long examId, Long courseId, String studentCode) {
        QueryWrapper<BasicExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicExamStudent::getExamId, examId)
                .eq(BasicExamStudent::getCourseId, courseId)
                .eq(BasicExamStudent::getStudentCode, studentCode);
        return this.getOne(queryWrapper);
    }
}
