package com.qmth.teachcloud.mark.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.common.bean.dto.MarkTaskSmsDto;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.MessageEnum;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.mark.bean.marker.MarkerAddParam;
import com.qmth.teachcloud.mark.dto.mark.entrance.MarkEntranceDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkQualityChartDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkQualityDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkUserGroupProgressDto;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;

import java.util.List;

/**
 * <p>
 * 评卷员表 服务类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
public interface MarkUserQuestionService extends IService<MarkUserQuestion> {

    IPage<MarkEntranceDto> listEntranceGroup(Long examId, Long openCollegeId, Long courseCode, String paperNumber,
                                             Integer pageNumber, Integer pageSize);

    List<MarkUser> listGroupUserByExamIdAndPaperNumberAndQuestionIdAndClassName(Long examId, String paperNumber, Long questionId, String className);

    IPage<MarkUserGroupProgressDto> pageGroupUserByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber,
                                                                                     Long questionId, String loginName, String className, Boolean marked, Integer pageNumber, Integer pageSize);

    void addMarkUserQuestion(MarkerAddParam markerAddParam);

    void setTopCount(List<Long> markUserQuestionIds, Integer topCount);

    MarkUserQuestion getByExamIdAndPaperNumberAndQuestionIdAndUserId(Long examId, String paperNumber, Long questionId, Long userId);

    void deleteMarker(MarkUserQuestion markUserQuestion);

    void releaseMarker(List<Long> markUserGroupIds);

    boolean resetMarker(MarkUserQuestion markUserGroup);

    void resetById(Long id);

    IPage<MarkQualityDto> pageQuality(Long examId, String paperNumber, Long questionId, String loginName, Integer pageNumber, Integer pageSize);

    void updateQuality(Long examId, String paperNumber, Long questionId);

    List<MarkUserQuestion> listByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId);

    void updateQualityById(Long markUserGroupId, int finishCount, int headerFinishCount, int validCount, double v,
                           double avgScore, double stdevScore, double maxScore, double minScore);

    List<MarkQualityChartDto> listQualityChart(Long examId, String paperNumber, Long questionId);

    List<MarkUserQuestion> listByExamIdAndPaperNumberAndUserIdAndEnableTure(Long examId, String paperNumber, Long userId);

    void deleteByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId);

    List<String> listBindMarkers(Long examId, String paperNumber, Long questionId);

    /**
     * 查询评卷任务待办/已办数量
     *
     * @param userId 用户id
     * @param status 试卷状态
     * @return 任务数量
     */
    int countByMarkTask(Long userId, MarkPaperStatus status);

    void updateRejectCountByExamIdAndPaperNumberAndQuestionIdAndUserId(Long examId, String paperNumber, Long questionId, Long userId);

    void saveDefaultMarker(Long examId, String paperNumber);

    List<String> countClassByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId);

    List<MarkQuestion> listSubjectiveQuestionByUserId(Long examId, String paperNumber, Long id);

    int countByExamIdAndPaperNumberAndAndUserId(Long examId, String paperNumber, Long userId);

    /**
     * 根据考试id&课程id&试卷编码查询待发送阅卷待办任务
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @return
     */
    List<MarkTaskSmsDto> findMarkTaskSms(Long examId, Long courseId, String paperNumber);

    /**
     * 短信count
     *
     * @param userId
     * @param mobileNumber
     * @param condition
     * @return
     */
    int smsConditionCount(Long userId, String mobileNumber, String condition);

    /**
     * 保存message和备注
     *
     * @param schoolId
     * @param userId
     * @param userName
     * @param mobileNumber
     * @param paperNumber
     * @param courseCode
     * @param variableParams
     * @param messageType
     * @param createId
     * @param remark
     */
    void saveMessageSendLogRemark(Long schoolId, Long userId, String userName, String mobileNumber, String paperNumber, String courseCode, String variableParams, MessageEnum messageType, Long createId, String remark);

    /**
     * 发送阅卷待办短信
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     */
    void smsMarkTask(Long examId, Long courseId, String paperNumber);

    List<MarkUser> listUserByExamIdAndPaperNumber(Long examId, String paperNumber);

    void updateEnableByExamIdAndPaperNumberAndQuestionIds(Long examId, String paperNumber, List<Long> questionIds, Boolean enable);

    void removeByExamIdAndPaperNumberAndQuestionIdsAndUserId(Long examId, String paperNumber, List<Long> questionIds, Long userId);
}
