package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.dto.mark.manage.TaskQuestion;
import com.qmth.teachcloud.mark.dto.mark.score.StudentObjectiveAnswerDto;
import com.qmth.teachcloud.mark.entity.MarkArchiveStudent;
import com.qmth.teachcloud.mark.mapper.MarkArchiveStudentMapper;
import com.qmth.teachcloud.mark.service.MarkArchiveStudentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 考生归档数据表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2025-03-21
 */
@Service
public class MarkArchiveStudentServiceImpl extends ServiceImpl<MarkArchiveStudentMapper, MarkArchiveStudent> implements MarkArchiveStudentService {

    @Override
    public List<MarkArchiveStudent> listByStudentIds(List<Long> studentIds) {
        QueryWrapper<MarkArchiveStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(MarkArchiveStudent::getStudentId, studentIds);
        return this.list(queryWrapper);
    }

    @Override
    public List<TaskQuestion> listTaskQuestionByStudentIds(List<Long> studentIds) {
        List<MarkArchiveStudent> markArchiveStudents = this.listByStudentIds(studentIds);
        List<TaskQuestion> taskQuestions = markArchiveStudents.stream()
                .filter(m -> StringUtils.isNotBlank(m.getSubjectiveQuestions()))
                .map(m -> JSON.parseArray(m.getSubjectiveQuestions(), TaskQuestion.class))
                .flatMap(List::stream).collect(Collectors.toList());
        return taskQuestions;
    }

    @Override
    public List<StudentObjectiveAnswerDto> listStudentObjectiveAnswerDtoByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkArchiveStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkArchiveStudent::getExamId, examId)
                .eq(MarkArchiveStudent::getPaperNumber, paperNumber)
                .isNotNull(MarkArchiveStudent::getObjectiveQuestions)
                .last(" limit 1");
        MarkArchiveStudent markArchiveStudent = this.getOne(queryWrapper);
        if (markArchiveStudent != null) {
            return JSON.parseArray(markArchiveStudent.getObjectiveQuestions(), StudentObjectiveAnswerDto.class);
        }
        return Collections.emptyList();
    }

    @Override
    public List<TaskQuestion> listTaskQuestionByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkArchiveStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkArchiveStudent::getExamId, examId)
                .eq(MarkArchiveStudent::getPaperNumber, paperNumber)
                .isNotNull(MarkArchiveStudent::getSubjectiveQuestions)
                .last(" limit 1");
        MarkArchiveStudent markArchiveStudent = this.getOne(queryWrapper);
        if (markArchiveStudent != null) {
            return JSON.parseArray(markArchiveStudent.getSubjectiveQuestions(), TaskQuestion.class);
        }
        return Collections.emptyList();
    }
}
