package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.mark.bean.archivescore.QuestionVo;
import com.qmth.teachcloud.mark.bean.dto.MarkAreaDto;
import com.qmth.teachcloud.mark.dto.mark.manage.Task;
import com.qmth.teachcloud.mark.dto.mark.manage.TaskQuestion;
import com.qmth.teachcloud.mark.entity.MarkArchiveStudent;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkSubjectiveScore;
import com.qmth.teachcloud.mark.mapper.MarkSubjectiveScoreMapper;
import com.qmth.teachcloud.mark.service.MarkArchiveStudentService;
import com.qmth.teachcloud.mark.service.MarkSubjectiveScoreService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.security.Escape;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 主观题得分明细表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkSubjectiveScoreServiceImpl extends MppServiceImpl<MarkSubjectiveScoreMapper, MarkSubjectiveScore> implements MarkSubjectiveScoreService {

    @Resource
    private MarkArchiveStudentService markArchiveStudentService;

    @Override
    public MarkSubjectiveScore getByStudentIdAndQuestionId(Long studentId, Long questionId) {
        QueryWrapper<MarkSubjectiveScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkSubjectiveScore::getStudentId, studentId)
                .eq(MarkSubjectiveScore::getQuestionId, questionId);
        return this.getOne(queryWrapper);
    }

    @Override
    public Set<Integer> listMainNumberByStudentIdAndUncalculate(Long studentId, boolean uncalculate) {
        Set<Integer> mainNumbers = new HashSet<Integer>();
        List<MarkSubjectiveScore> list = this.listByStudentIdAndUncalculate(studentId, uncalculate);
        for (MarkSubjectiveScore subjectiveScore : list) {
            mainNumbers.add(subjectiveScore.getMainNumber());
        }
        return mainNumbers;
    }

    @Override
    public List<MarkSubjectiveScore> listByStudentIdAndUncalculate(Long studentId, boolean uncalculate) {
        QueryWrapper<MarkSubjectiveScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkSubjectiveScore::getStudentId, studentId)
                .orderByAsc(MarkSubjectiveScore::getMainNumber)
                .orderByAsc(MarkSubjectiveScore::getSubNumber);
        return this.list(queryWrapper);
    }

    @Override
    public void deleteByStudentId(Long studentId) {
        UpdateWrapper<MarkSubjectiveScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkSubjectiveScore::getStudentId, studentId);
        this.remove(updateWrapper);
    }

    @Override
    public List<QuestionVo> getSubjectiveVo(MarkPaper markPaper, List<Long> studentIds) {
        if (CollectionUtils.isEmpty(studentIds)) {
            return null;
        }
        if (markPaper.getArchive()) {
            List<TaskQuestion> taskQuestionList = markArchiveStudentService.listTaskQuestionByStudentIds(studentIds);
            Map<String, List<TaskQuestion>> taskQuestions = taskQuestionList.stream().collect(Collectors.groupingBy(m -> m.getMainNumber() + SystemConstant.DEFAULT_SIGN + m.getSubNumber()));
            List<QuestionVo> questionVos = new ArrayList<>();
            for (Map.Entry<String, List<TaskQuestion>> entry : taskQuestions.entrySet()) {
                String[] keys = entry.getKey().split(SystemConstant.DEFAULT_SIGN);
                List<TaskQuestion> valueList = entry.getValue();
                TaskQuestion taskQuestion = valueList.get(0);
                QuestionVo questionVo = new QuestionVo();
                questionVo.setTitle(taskQuestion.getTitle());
                questionVo.setMainNumber(Integer.valueOf(keys[0]));
                questionVo.setSubNumber(Integer.valueOf(keys[1]));
                questionVo.setScore(taskQuestion.getMaxScore());
                questionVo.setStudentCount(valueList.size());
                Double collect = valueList.stream().collect(Collectors.averagingDouble(TaskQuestion::getMarkerScore));
                questionVo.setAvgScore(collect);
                long count = valueList.stream().filter(m -> m.getMarkerScore() > 0).count();
                questionVo.setScoreCount(Math.toIntExact(count));
                long fullCount = valueList.stream().filter(m -> m.getMarkerScore() - m.getMaxScore() == 0).count();
                questionVo.setFullScoreCount(Math.toIntExact(fullCount));
                questionVos.add(questionVo);
            }
            questionVos.sort(Comparator.comparing(QuestionVo::getMainNumber).thenComparing(QuestionVo::getSubNumber));
            return questionVos;

        } else {
            return this.baseMapper.getSubjectiveVo(studentIds);
        }
    }

    @Override
    public List<MarkSubjectiveScore> listByStudentId(Long studentId) {
        QueryWrapper<MarkSubjectiveScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkSubjectiveScore::getStudentId, studentId)
                .orderByAsc(MarkSubjectiveScore::getMainNumber)
                .orderByAsc(MarkSubjectiveScore::getSubNumber);
        return this.list(queryWrapper);
    }

    @Override
    public void updateRejected(Long studentId, Long questionId, boolean rejectd) {
        UpdateWrapper<MarkSubjectiveScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkSubjectiveScore::getRejected, rejectd)
                .eq(MarkSubjectiveScore::getStudentId, studentId)
                .eq(MarkSubjectiveScore::getQuestionId, questionId);
        this.update(updateWrapper);
    }

    @Override
    public long countByStudentId(Long studentId) {
        QueryWrapper<MarkSubjectiveScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkSubjectiveScore::getStudentId, studentId);
        return this.count(queryWrapper);
    }

    @Override
    public void deleteByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId) {
        UpdateWrapper<MarkSubjectiveScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkSubjectiveScore::getExamId, examId)
                .eq(MarkSubjectiveScore::getPaperNumber, paperNumber)
                .eq(MarkSubjectiveScore::getQuestionId, questionId);
        this.remove(updateWrapper);
    }

    /**
     * 根据学生id和考试id以及试卷编码查询
     *
     * @param studentId
     * @param examId
     * @param paperNumber
     * @return
     */
    @Override
    public List<MarkSubjectiveScore> listByStudentIdAndExamIdAndPn(Long studentId, Long examId, String paperNumber) {
        QueryWrapper<MarkSubjectiveScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkSubjectiveScore::getStudentId, studentId)
                .eq(MarkSubjectiveScore::getExamId, examId)
                .eq(MarkSubjectiveScore::getPaperNumber, paperNumber)
                .orderByAsc(MarkSubjectiveScore::getMainNumber)
                .orderByAsc(MarkSubjectiveScore::getSubNumber);
        return this.list(queryWrapper);
    }

    /**
     * 根据学生id查找评卷区
     *
     * @param studentId
     * @return
     */
    @Override
    public List<MarkAreaDto> findMarkArea(Long studentId) {
        return this.baseMapper.findMarkArea(studentId);
    }

    @Override
    public void deleteByStudentIdAndQuestionId(Long studentId, Long questionId) {
        UpdateWrapper<MarkSubjectiveScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkSubjectiveScore::getStudentId, studentId)
                .eq(MarkSubjectiveScore::getQuestionId, questionId);
        this.remove(updateWrapper);
    }
}
