package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.Update;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.ImmutableMap;
import com.qmth.teachcloud.common.bean.dto.MarkTaskSmsDto;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.common.bean.result.SmsResponseResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.MessageEnum;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.service.SysConfigService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.common.util.SmsSendUtil;
import com.qmth.teachcloud.mark.bean.marker.MarkerAddParam;
import com.qmth.teachcloud.mark.dto.BasicMessageDto;
import com.qmth.teachcloud.mark.dto.mark.entrance.MarkEntranceDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkQualityChartDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkQualityDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkUserGroupProgressDto;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.entity.MarkUserClass;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.mapper.MarkUserQuestionMapper;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.Calculator;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.filefilter.FalseFileFilter;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.jsf.FacesContextUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 评卷员表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkUserQuestionServiceImpl extends ServiceImpl<MarkUserQuestionMapper, MarkUserQuestion> implements MarkUserQuestionService {

    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkUserClassService markUserClassService;
    @Resource
    private MarkSyncService markSyncService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private MarkService markService;
    @Resource
    private LockService lockService;

    @Resource
    private SmsSendUtil smsSendUtil;

    @Resource
    private SysConfigService sysConfigService;

    @Override
    public IPage<MarkEntranceDto> listEntranceGroup(Long examId, Long openCollegeId, Long courseId, String paperNumber, Integer pageNumber, Integer pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<MarkUserQuestion> page = new Page<>(pageNumber, pageSize);
        IPage<MarkEntranceDto> markEntranceDtoIPage = this.baseMapper.listEntranceGroup(page, examId, openCollegeId, courseId, paperNumber, sysUser.getId(), MarkPaperStatus.FORMAL.name());
        for (MarkEntranceDto record : markEntranceDtoIPage.getRecords()) {
            MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(record.getExamId(), record.getPaperNumber());
            // 分班阅
            List<String> classNames = null;
            if (markPaper != null && markPaper.getClassMark() != null && markPaper.getClassMark()) {
                List<MarkUserClass> markUserClassList = markUserClassService.listByExamIdAndPaperNumberAndUserId(record.getExamId(), record.getPaperNumber(), sysUser.getId());
                if (CollectionUtils.isNotEmpty(markUserClassList)) {
                    classNames = markUserClassList.stream().map(MarkUserClass::getClassName).collect(Collectors.toList());
                }
            }

            List<MarkQuestion> markQuestionList = this.listSubjectiveQuestionByUserId(record.getExamId(), record.getPaperNumber(), sysUser.getId());
            List<Long> questionIds = markQuestionList.stream().map(MarkQuestion::getId).collect(Collectors.toList());

            int taskCount = markTaskService.countByExamIdAndPaperNumberAndUserIdAndAndClassNameAndQuestionIdIn(record.getExamId(), record.getPaperNumber(), null, classNames, questionIds);
            record.setTaskCount(taskCount);
            int unMarkedCount = markTaskService.countByExamIdAndPaperNumberAndUserIdAndAndClassNameAndQuestionIdIn(record.getExamId(), record.getPaperNumber(), null, classNames, questionIds, MarkTaskStatus.WAITING, MarkTaskStatus.WAIT_ARBITRATE, MarkTaskStatus.REJECTED, MarkTaskStatus.PROBLEM);
            record.setLeftCount(unMarkedCount);
            record.setMarkedCount(taskCount == 0 ? 0 : taskCount - unMarkedCount);
            // 评卷进度
            if (taskCount == 0) {
                record.setPercent(0D);
            } else {
                BigDecimal bigDecimal = new BigDecimal(record.getMarkedCount() * 100).divide(new BigDecimal(taskCount), 2, RoundingMode.HALF_UP);
                record.setPercent(bigDecimal.doubleValue());
            }
            // 分组题目
            record.setQuestionNumber(markQuestionList.stream().map(m -> m.getMainNumber() + "-" + m.getSubNumber()).collect(Collectors.joining(",")));
        }

        return markEntranceDtoIPage;
    }

    @Override
    public List<MarkUser> listGroupUserByExamIdAndPaperNumberAndQuestionIdAndClassName(Long examId, String paperNumber, Long questionId, String className) {
        return this.baseMapper.listGroupUserByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId, className);
    }

    @Override
    public IPage<MarkUserGroupProgressDto> pageGroupUserByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId, String loginName, String className, Boolean marked, Integer pageNumber, Integer pageSize) {
        Page<MarkUserGroupProgressDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkUserGroupProgressDto> markUserGroupProgressDtoIPage = this.baseMapper.pageGroupUserByExamIdAndPaperNumberAndQuestionId(page, examId, paperNumber, questionId, loginName, className, marked);
        for (MarkUserGroupProgressDto record : markUserGroupProgressDtoIPage.getRecords()) {
            MarkQuestion markQuestion = markQuestionService.getById(record.getQuestionId());
            record.setQuestionNumber(markQuestion.getQuestionNumber());
            List<MarkTask> markTaskList = markTaskService.listByExamIdAndPaperNumberAndQuestionIdAndUserIdAndClassName(examId, paperNumber, record.getQuestionId(), record.getUserId(), null);
            record.setResetting(lockService.isLocked(LockType.MARKER_RESET, record.getMarkUserQuestionId()));
            record.setMarkedCount(markTaskService.markedCount(markTaskList));
            record.setCurrentCount(markService.applyCurrentCount(markQuestion, record.getUserId()));
        }
        return markUserGroupProgressDtoIPage;
    }

    @Transactional
    @Override
    public void addMarkUserQuestion(MarkerAddParam markerAddParam) {
        if (CollectionUtils.isNotEmpty(markerAddParam.getUserIds())) {
            // 根据是否合并设置评卷员参数，做合并
            List<Long> questionIds = listMergeQuestionId(markerAddParam.getExamId(), markerAddParam.getPaperNumber(), markerAddParam.getQuestionId());

            this.updateEnableByExamIdAndPaperNumberAndQuestionIds(markerAddParam.getExamId(), markerAddParam.getPaperNumber(), questionIds, false);
            List<MarkUserQuestion> markUserQuestionAdd = new ArrayList<>();
            List<MarkUserQuestion> markUserQuestionUpdate = new ArrayList<>();
            for (Long userId : markerAddParam.getUserIds()) {
                for (Long questionId : questionIds) {
                    MarkUserQuestion markUserGroup = this.getByExamIdAndPaperNumberAndQuestionIdAndUserId(markerAddParam.getExamId(), markerAddParam.getPaperNumber(), questionId, userId);
                    if (markUserGroup == null) {
                        markUserQuestionAdd.add(new MarkUserQuestion(markerAddParam.getExamId(), markerAddParam.getPaperNumber(), questionId, userId));
                    } else if (!markUserGroup.getEnable()) {
                        markUserGroup.setEnable(true);
                        markUserQuestionUpdate.add(markUserGroup);
                    }
                }
            }
            if (CollectionUtils.isNotEmpty(markUserQuestionAdd)) {
                this.saveBatch(markUserQuestionAdd);
            }
            if (CollectionUtils.isNotEmpty(markUserQuestionUpdate)) {
                this.saveOrUpdateBatch(markUserQuestionUpdate);
            }
        }

        List<MarkUserQuestion> markUserQuestionList = this.listByExamIdAndPaperNumberAndQuestionId(markerAddParam.getExamId(), markerAddParam.getPaperNumber(), markerAddParam.getQuestionId());
        for (MarkUserQuestion markUserQuestion : markUserQuestionList) {
            if (markUserQuestion.getEnable()) {
                continue;
            }
            this.deleteMarker(markUserQuestion);
        }

//        markPaperService.updateGroupStatus(markerAddParam.getExamId(), markerAddParam.getPaperNumber());
    }

    private List<Long> listMergeQuestionId(Long examId, String paperNumber, Long questionId) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);

        MarkQuestion markQuestion = markQuestionService.getById(questionId);
        if (markQuestion == null) {
            throw ExceptionResultEnum.ERROR.exception("题目不存在");
        }
        List<Long> questionIds = new ArrayList<>();
        if (markQuestion.getQuestionType() == 4 && markPaper != null && markPaper.getMergeMarker() != null && markPaper.getMergeMarker()) {
            List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
            // 填空题，整个大题一起解绑
            return markQuestionList.stream().filter(m -> markQuestion.getMainNumber().equals(m.getMainNumber()) && m.getQuestionType() == 4).map(MarkQuestion::getId).collect(Collectors.toList());
        } else {
            questionIds.add(questionId);
        }
        return questionIds;
    }

    @Override
    public void setTopCount(List<Long> markUserQuestionIds, Integer topCount) {
        for (Long markUserQuestionId : markUserQuestionIds) {
            MarkUserQuestion markUserQuestion = this.getById(markUserQuestionId);
            if (markUserQuestion != null) {
                markUserQuestion.setTopCount(topCount);
                this.updateById(markUserQuestion);
            }
        }
    }

    @Override
    public MarkUserQuestion getByExamIdAndPaperNumberAndQuestionIdAndUserId(Long examId, String paperNumber, Long questionId, Long userId) {
        QueryWrapper<MarkUserQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserQuestion::getExamId, examId)
                .eq(MarkUserQuestion::getPaperNumber, paperNumber)
                .eq(MarkUserQuestion::getQuestionId, questionId)
                .eq(MarkUserQuestion::getUserId, userId);
        return this.getOne(queryWrapper);
    }

    @Override
    public void deleteMarker(MarkUserQuestion markUserQuestion) {
        // 先回收，再删除
        releaseMarker(Arrays.asList(markUserQuestion.getId()));

        List<Long> ids = listMergeQuestionId(markUserQuestion.getExamId(), markUserQuestion.getPaperNumber(), markUserQuestion.getQuestionId());

        List<MarkTask> markTaskList = markTaskService.listByExamIdAndPaperNumberAndQuestionIdAndUserIdAndClassName(markUserQuestion.getExamId(), markUserQuestion.getPaperNumber(), markUserQuestion.getQuestionId(), markUserQuestion.getUserId(), null);
        if (markTaskService.markedCount(markTaskList) > 0) {
            // 逻辑解绑
            UpdateWrapper<MarkUserQuestion> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(MarkUserQuestion::getEnable, false)
                    .eq(MarkUserQuestion::getUserId, markUserQuestion.getUserId())
                    .in(MarkUserQuestion::getQuestionId, ids);
            this.update(updateWrapper);
        } else {
            this.removeByExamIdAndPaperNumberAndQuestionIdsAndUserId(markUserQuestion.getExamId(), markUserQuestion.getPaperNumber(), ids, markUserQuestion.getUserId());
        }

        markPaperService.updateGroupStatus(markUserQuestion.getExamId(), markUserQuestion.getPaperNumber());
    }

    @Override
    public void releaseMarker(List<Long> markUserGroupIds) {
        for (Long markUserGroupId : markUserGroupIds) {
            MarkUserQuestion markUserGroup = this.getById(markUserGroupId);
            if (markUserGroup != null) {
                try {
                    lockService.waitlock(LockType.MARK_USER_QUESTION, markUserGroupId);
                    markService.releaseByMarkUserGroup(markUserGroup);
                } catch (Exception e) {
                    log.error("release marker error", e);
                } finally {
                    lockService.unlock(LockType.MARK_USER_QUESTION, markUserGroupId);
                }
            }
        }
    }

    @Override
    public boolean resetMarker(MarkUserQuestion markUserQuestion) {
        if (markUserQuestion != null) {
            if (lockService.trylock(LockType.MARKER_RESET, markUserQuestion.getId())) {
                markSyncService.markerResetSync(markUserQuestion);
                return true;
            } else {
                throw ExceptionResultEnum.ERROR.exception("评卷员正在重置");
            }
        }
        return true;
    }

    @Override
    public void resetById(Long id) {
        UpdateWrapper<MarkUserQuestion> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkUserQuestion::getFinishCount, null)
                .set(MarkUserQuestion::getHeaderFinishCount, null)
                .set(MarkUserQuestion::getValidCount, null)
                .set(MarkUserQuestion::getAvgScore, null)
                .set(MarkUserQuestion::getAvgSpeed, null)
                .set(MarkUserQuestion::getStdevScore, null)
                .set(MarkUserQuestion::getMaxScore, null)
                .set(MarkUserQuestion::getMinScore, null)
                .eq(MarkUserQuestion::getId, id);
        this.update(updateWrapper);
    }

    @Override
    public IPage<MarkQualityDto> pageQuality(Long examId, String paperNumber, Long questionId, String loginName, Integer pageNumber, Integer pageSize) {
        Page<MarkQualityDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkQualityDto> markQualityDtoIPage = this.baseMapper.pageQuality(page, examId, paperNumber, questionId, loginName);
        for (MarkQualityDto qualityDto : markQualityDtoIPage.getRecords()) {
            MarkQuestion markQuestion = markQuestionService.getById(qualityDto.getQuestionId());
            if (markQuestion != null) {
                qualityDto.setQuestionNumber(markQuestion.getQuestionNumber());
            }
            if (qualityDto.getUserId() != null) {
                SysUser sysUser = sysUserService.getByUserId(qualityDto.getUserId());
                qualityDto.setLoginName(sysUser.getLoginName());
                qualityDto.setName(sysUser.getRealName());
                qualityDto.setOrgName(sysUser.getOrgName());
                qualityDto.setAdoptionRate(Calculator.divide2String(Calculator.multiply(qualityDto.getValidCount(), 100), Double.valueOf(qualityDto.getFinishCount()), 2));
                qualityDto.setAvgScore(Calculator.round(qualityDto.getAvgScore(), 2));
            }
        }
        return markQualityDtoIPage;
    }

    @Override
    public void updateQuality(Long examId, String paperNumber, Long questionId) {
        if (questionId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择评阅题目");
        }
        MarkQuestion markQuestion = markQuestionService.getById(questionId);
        if (markQuestion != null) {
            String lockKey = markService.getGroupKey(markQuestion);
            if (lockService.trylock(LockType.BATCH_QUALITY, lockKey)) {
                markSyncService.updateQuality(this.listByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId), lockKey);
            }
        }
    }

    @Override
    public List<MarkUserQuestion> listByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId) {
        QueryWrapper<MarkUserQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserQuestion::getExamId, examId)
                .eq(MarkUserQuestion::getPaperNumber, paperNumber)
                .eq(MarkUserQuestion::getQuestionId, questionId)
                .orderByAsc(MarkUserQuestion::getId);
        return this.list(queryWrapper);
    }

    @Override
    public void updateQualityById(Long markUserGroupId, int finishCount, int headerFinishCount, int validCount, double avgSpeed, double avgScore, double stdevScore, double maxScore, double minScore) {
        UpdateWrapper<MarkUserQuestion> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkUserQuestion::getFinishCount, finishCount)
                .set(MarkUserQuestion::getHeaderFinishCount, headerFinishCount)
                .set(MarkUserQuestion::getValidCount, validCount)
                .set(MarkUserQuestion::getAvgSpeed, avgSpeed)
                .set(MarkUserQuestion::getAvgScore, avgScore)
                .set(MarkUserQuestion::getStdevScore, stdevScore)
                .set(MarkUserQuestion::getMaxScore, maxScore)
                .set(MarkUserQuestion::getMinScore, minScore)
                .eq(MarkUserQuestion::getId, markUserGroupId);
        this.update(updateWrapper);
    }

    @Override
    public List<MarkQualityChartDto> listQualityChart(Long examId, String paperNumber, Long questionId) {
        if (questionId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择对应的科目及题目，再查看给分曲线");
        }
        List<MarkQualityChartDto> markQualityChartDtoList = new ArrayList<>();
        List<MarkUserQuestion> markUserQuestionList = this.listByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId);
        Double totalAvgScore = markUserQuestionList.stream().filter(m -> m.getAvgScore() != null).collect(Collectors.averagingDouble(MarkUserQuestion::getAvgScore));
        for (MarkUserQuestion markUserQuestion : markUserQuestionList) {
            MarkQualityChartDto markQualityChartDto = new MarkQualityChartDto();
            markQualityChartDto.setUserId(markUserQuestion.getUserId());
            SysUser sysUser = sysUserService.getById(markUserQuestion.getUserId());
            markQualityChartDto.setLoginName(sysUser.getLoginName());
            markQualityChartDto.setName(sysUser.getRealName());
            markQualityChartDto.setTotalAvgScore(Calculator.round(totalAvgScore, 2));
            markQualityChartDto.setAvgScore(markUserQuestion.getAvgScore() == null ? null : Calculator.round(markUserQuestion.getAvgScore(), 2));
            markQualityChartDto.setMaxScore(markUserQuestion.getMaxScore() == null ? null : Calculator.round(markUserQuestion.getMaxScore(), 2));
            markQualityChartDto.setMinScore(markUserQuestion.getMinScore() == null ? null : Calculator.round(markUserQuestion.getMinScore(), 2));
            markQualityChartDtoList.add(markQualityChartDto);
        }
        return markQualityChartDtoList;
    }

    @Override
    public List<MarkUserQuestion> listByExamIdAndPaperNumberAndUserIdAndEnableTure(Long examId, String paperNumber, Long userId) {
        QueryWrapper<MarkUserQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserQuestion::getExamId, examId)
                .eq(MarkUserQuestion::getPaperNumber, paperNumber)
                .eq(MarkUserQuestion::getUserId, userId)
                .eq(MarkUserQuestion::getEnable, true);
        return this.list(queryWrapper);
    }

    @Override
    public void deleteByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId) {
        UpdateWrapper<MarkUserQuestion> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkUserQuestion> lambda = updateWrapper.lambda();
        lambda.eq(MarkUserQuestion::getExamId, examId)
                .eq(MarkUserQuestion::getPaperNumber, paperNumber);
        if (questionId != null) {
            lambda.eq(MarkUserQuestion::getQuestionId, questionId);
        }
        this.remove(updateWrapper);
    }

    @Override
    public List<String> listBindMarkers(Long examId, String paperNumber, Long questionId) {
        List<MarkUserQuestion> markUserQuestions = listByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId);
        List<String> userIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(markUserQuestions)) {
            userIds = markUserQuestions.stream().filter(m -> m.getEnable()).map(m -> String.valueOf(m.getUserId())).collect(Collectors.toList());
        }
        return userIds;
    }

    @Override
    public int countByMarkTask(Long userId, MarkPaperStatus status) {
        return this.baseMapper.countByMarkTask(userId, status.name());
    }

    @Override
    public void updateRejectCountByExamIdAndPaperNumberAndQuestionIdAndUserId(Long examId, String paperNumber, Long questionId, Long userId) {
        this.baseMapper.updateRejectCountByExamIdAndPaperNumberAndQuestionIdAndUserId(examId, paperNumber, questionId, userId);
    }

    @Override
    public void saveDefaultMarker(Long examId, String paperNumber) {
        List<MarkQuestion> markQuestions = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
        if (CollectionUtils.isEmpty(markQuestions)) {
            return;
        }
        List<SysUser> sysUserList = this.baseMapper.listDefaultMarkerByExamIdAndPaperNumber(examId, paperNumber);
        List<MarkUserQuestion> markUserQuestions = new ArrayList<>();
        for (SysUser sysUser : sysUserList) {
            for (MarkQuestion markQuestion : markQuestions) {
                MarkUserQuestion markUserQuestion = this.getByExamIdAndPaperNumberAndQuestionIdAndUserId(examId, paperNumber, markQuestion.getId(), sysUser.getId());
                if (markUserQuestion == null) {
                    markUserQuestions.add(new MarkUserQuestion(examId, paperNumber, markQuestion.getId(), sysUser.getId()));
                }
            }
        }
        this.saveBatch(markUserQuestions);
    }

    @Override
    public List<String> countClassByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId) {
        return this.baseMapper.countClassByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId);
    }

    @Override
    public List<MarkQuestion> listSubjectiveQuestionByUserId(Long examId, String paperNumber, Long userId) {
        return this.baseMapper.listSubjectiveQuestionByUserId(examId, paperNumber, userId);
    }

    @Override
    public int countByExamIdAndPaperNumberAndAndUserId(Long examId, String paperNumber, Long userId) {
        QueryWrapper<MarkUserQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserQuestion::getExamId, examId)
                .eq(MarkUserQuestion::getPaperNumber, paperNumber)
                .eq(MarkUserQuestion::getUserId, userId);
        return this.count(queryWrapper);
    }

    /**
     * 根据考试id&课程id&试卷编码查询待发送阅卷待办任务
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @return
     */
    @Override
    public List<MarkTaskSmsDto> findMarkTaskSms(Long examId, Long courseId, String paperNumber) {
        return this.baseMapper.findMarkTaskSms(examId, courseId, paperNumber);
    }

    /**
     * 短信count
     *
     * @param userId
     * @param mobileNumber
     * @param condition
     * @return
     */
    @Override
    public int smsConditionCount(Long userId, String mobileNumber, String condition) {
        return this.baseMapper.smsConditionCount(userId, mobileNumber, condition);
    }

    /**
     * 保存message和备注
     *
     * @param schoolId
     * @param userId
     * @param userName
     * @param mobileNumber
     * @param paperNumber
     * @param courseCode
     * @param variableParams
     * @param messageType
     * @param createId
     * @param remark
     */
    @Override
    @Transactional
    public void saveMessageSendLogRemark(Long schoolId, Long userId, String userName, String mobileNumber, String paperNumber, String courseCode, String variableParams, MessageEnum messageType, Long createId, String remark) {
        BasicMessageDto basicMessageDto = null;
        String templateCode = null;
        try {
            basicMessageDto = new BasicMessageDto(schoolId, userId, userName, mobileNumber, paperNumber,
                    courseCode, variableParams, messageType, createId,
                    remark, templateCode);
            // code和content
            Map<String, String> enumInfo = smsSendUtil.getCodeAndContentByEnum(messageType);
            if (!enumInfo.containsKey("templateCode")) {
                throw ExceptionResultEnum.ERROR.exception("未找到短信模板Code");
            }
            templateCode = enumInfo.get("templateCode");
            if (StringUtils.isBlank(templateCode)) {
                throw ExceptionResultEnum.ERROR.exception("阿里云短信模板Code必填");
            }

            // 发送短信参数配置验证
            SysConfig sysConfig = sysConfigService.getByKey("sys.message.enable");
            if (sysConfig == null) {
                throw ExceptionResultEnum.ERROR.exception("未找到发送短信开关配置参数，默认不发送");
            }
            if (sysConfig.getConfigValue() == null) {
                throw ExceptionResultEnum.ERROR.exception("未正确配置发送短信开关参数，默认不发送");
            }
            if (sysConfig.getConfigValue().equals("false")) {
                throw ExceptionResultEnum.ERROR.exception("发送短信开关设置为已关闭");
            }

            // 参数校验
            if (StringUtils.isBlank(userName)) {
                throw ExceptionResultEnum.ERROR.exception("用户名称必填");
            }
            if (StringUtils.isBlank(mobileNumber)) {
                throw ExceptionResultEnum.ERROR.exception("用户手机号必填");
            }
            if (StringUtils.isBlank(variableParams)) {
                throw ExceptionResultEnum.ERROR.exception("短信内容参数值必填");
            }

            // 调用阿里云短信平台发送短信
            Map<String, Object> templateParam = JSON.parseObject(variableParams, Map.class);
            SmsResponseResult smsResponseResult = smsSendUtil.sendSms(mobileNumber, templateCode, templateParam);
            if (!SmsSendUtil.OK.equals(smsResponseResult.getCode())) {
                throw ExceptionResultEnum.ERROR.exception("阿里云短信发送接口调用失败");
            }
            basicMessageDto.setSendStatus(smsResponseResult.getCode());
            basicMessageDto.setSendResult(smsResponseResult.getMessage());

        } catch (Exception e) {
            basicMessageDto.setErrorInfo(e.getMessage());
        } finally {
            this.baseMapper.insertSms(basicMessageDto);
        }
    }

    /**
     * 发送阅卷待办短信
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     */
    @Override
    @Transactional
    public void smsMarkTask(Long examId, Long courseId, String paperNumber) {
        List<MarkTaskSmsDto> markTaskSmsDtoList = this.findMarkTaskSms(examId, courseId, paperNumber);
        if (CollectionUtils.isNotEmpty(markTaskSmsDtoList)) {
            for (MarkTaskSmsDto m : markTaskSmsDtoList) {
                if(StringUtils.isBlank(m.getMobileNumber())){
                    continue;
                }
                StringJoiner stringJoiner = new StringJoiner("-");
                stringJoiner.add(m.getExamId().toString()).add(m.getCourseId().toString()).add(m.getPaperNumber());
                int count = this.smsConditionCount(m.getUserId(), m.getMobileNumber(), stringJoiner.toString());
                if (count == 0) {
                    this.saveMessageSendLogRemark(m.getSchoolId(), m.getUserId(), m.getRealName(), m.getMobileNumber(), m.getPaperNumber(), null, JSON.toJSONString(ImmutableMap.of("courseName", m.getCourseName(), "examName", m.getExamName())), MessageEnum.NOTICE_OF_MARK_TASK_CREATED, -1L, stringJoiner.toString());
                }
            }
        }
    }

    @Override
    public List<MarkUser> listUserByExamIdAndPaperNumber(Long examId, String paperNumber) {
        return this.baseMapper.listUserByExamIdAndPaperNumber(examId, paperNumber);
    }

    @Override
    public void updateEnableByExamIdAndPaperNumberAndQuestionIds(Long examId, String paperNumber, List<Long> questionIds, Boolean enable) {
        UpdateWrapper<MarkUserQuestion> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkUserQuestion::getEnable, enable)
                .eq(MarkUserQuestion::getExamId, examId)
                .eq(MarkUserQuestion::getPaperNumber, paperNumber)
                .in(MarkUserQuestion::getQuestionId, questionIds);
        this.update(updateWrapper);
    }

    @Override
    public void removeByExamIdAndPaperNumberAndQuestionIdsAndUserId(Long examId, String paperNumber, List<Long> questionIds, Long userId) {
        UpdateWrapper<MarkUserQuestion> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkUserQuestion::getExamId, examId)
                .eq(MarkUserQuestion::getPaperNumber, paperNumber)
                .in(MarkUserQuestion::getQuestionId, questionIds)
                .eq(MarkUserQuestion::getUserId, userId);
        this.remove(updateWrapper);
    }
}
